@@ -3,7 +3,7 @@
 /***********************************************************************
 *  This code is part of GLPK (GNU Linear Programming Kit).
 *
-*  Copyright (C) 2000, 01, 02, 03, 04, 05, 06, 07 Andrew Makhorin,
+*  Copyright (C) 2000,01,02,03,04,05,06,07,08,2009 Andrew Makhorin,
 *  Department for Applied Informatics, Moscow Aviation Institute,
 *  Moscow, Russia. All rights reserved. E-mail: <mao@mai2.rcnet.ru>.
 *
@@ -21,38 +21,41 @@
 *  along with GLPK. If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/
 
+#define _GLPSTD_ERRNO
+#define _GLPSTD_STDIO
 #include "glphbm.h"
 #include "glplib.h"
-#define print xprint1
 
-/*----------------------------------------------------------------------
--- hbm_read_mat - read sparse matrix in Harwell-Boeing format.
---
--- *Synopsis*
---
--- #include "glphbm.h"
--- HBM *hbm_read_mat(char *fname);
---
--- *Description*
---
--- The routine hbm_read_mat reads a sparse matrix in the Harwell-Boeing
--- format from a text file whose name is the character string fname.
---
--- Detailed description of the Harwell-Boeing format recognised by this
--- routine is given in the following report:
---
--- I.S.Duff, R.G.Grimes, J.G.Lewis. User's Guide for the Harwell-Boeing
--- Sparse Matrix Collection (Release I), TR/PA/92/86, October 1992.
---
--- *Returns*
---
--- If no error occured, the routine hbm_read_mat returns a pointer to
--- a data structure containing the matrix. In case of error the routine
--- prints an appropriate error message and returns NULL. */
+/***********************************************************************
+*  NAME
+*
+*  hbm_read_mat - read sparse matrix in Harwell-Boeing format
+*
+*  SYNOPSIS
+*
+*  #include "glphbm.h"
+*  HBM *hbm_read_mat(const char *fname);
+*
+*  DESCRIPTION
+*
+*  The routine hbm_read_mat reads a sparse matrix in the Harwell-Boeing
+*  format from a text file whose name is the character string fname.
+*
+*  Detailed description of the Harwell-Boeing format recognised by this
+*  routine is given in the following report:
+*
+*  I.S.Duff, R.G.Grimes, J.G.Lewis. User's Guide for the Harwell-Boeing
+*  Sparse Matrix Collection (Release I), TR/PA/92/86, October 1992.
+*
+*  RETURNS
+*
+*  If no error occured, the routine hbm_read_mat returns a pointer to
+*  a data structure containing the matrix. In case of error the routine
+*  prints an appropriate error message and returns NULL. */
 
 struct dsa
 {     /* working area used by routine hbm_read_mat */
-      char *fname;
+      const char *fname;
       /* name of input text file */
       FILE *fp;
       /* stream assigned to input text file */
@@ -72,12 +75,12 @@ struct dsa
       /* number of decimal places after point */
 };
 
-/*----------------------------------------------------------------------
--- read_card - read next data card.
---
--- This routine reads the next 80-column card from the input text file
--- and stores its image into the character string card. If the card was
--- read successfully, the routine returns zero, otherwise non-zero. */
+/***********************************************************************
+*  read_card - read next data card
+*
+*  This routine reads the next 80-column card from the input text file
+*  and stores its image into the character string card. If the card was
+*  read successfully, the routine returns zero, otherwise non-zero. */
 
 static int read_card(struct dsa *dsa)
 {     int k, c;
@@ -87,26 +90,29 @@ static int read_card(struct dsa *dsa)
       for (;;)
       {  c = fgetc(dsa->fp);
          if (ferror(dsa->fp))
-         {  print("%s:%d: read error - %s", dsa->fname, dsa->seqn,
+         {  xprintf("%s:%d: read error - %s\n", dsa->fname, dsa->seqn,
                strerror(errno));
             return 1;
          }
          if (feof(dsa->fp))
          {  if (k == 0)
-               print("%s:%d: unexpected EOF", dsa->fname, dsa->seqn);
+               xprintf("%s:%d: unexpected EOF\n", dsa->fname,
+                  dsa->seqn);
             else
-               print("%s:%d: missing final LF", dsa->fname, dsa->seqn);
+               xprintf("%s:%d: missing final LF\n", dsa->fname,
+                  dsa->seqn);
             return 1;
          }
          if (c == '\r') continue;
          if (c == '\n') break;
          if (iscntrl(c))
-         {  print("%s:%d: invalid control character 0x%02X", dsa->fname,
-               dsa->seqn, c);
+         {  xprintf("%s:%d: invalid control character 0x%02X\n",
+               dsa->fname, dsa->seqn, c);
             return 1;
          }
          if (k == 80)
-         {  print("%s:%d: card image too long", dsa->fname, dsa->seqn);
+         {  xprintf("%s:%d: card image too long\n", dsa->fname,
+               dsa->seqn);
             return 1;
          }
          dsa->card[k++] = (char)c;
@@ -114,14 +120,14 @@ static int read_card(struct dsa *dsa)
       return 0;
 }
 
-/*----------------------------------------------------------------------
--- scan_int - scan integer value from the current card.
---
--- This routine scans an integer value from the current card, where fld
--- is the name of the field, pos is the position of the field, width is
--- the width of the field, val points to a location to which the scanned
--- value should be stored. If the value was scanned successfully, the
--- routine returns zero, otherwise non-zero. */
+/***********************************************************************
+*  scan_int - scan integer value from the current card
+*
+*  This routine scans an integer value from the current card, where fld
+*  is the name of the field, pos is the position of the field, width is
+*  the width of the field, val points to a location to which the scanned
+*  value should be stored. If the value was scanned successfully, the
+*  routine returns zero, otherwise non-zero. */
 
 static int scan_int(struct dsa *dsa, char *fld, int pos, int width,
       int *val)
@@ -129,28 +135,28 @@ static int scan_int(struct dsa *dsa, char *fld, int pos, int width,
       xassert(1 <= width && width <= 80);
       memcpy(str, dsa->card + pos, width), str[width] = '\0';
       if (str2int(strspx(str), val))
-      {  print("%s:%d: field `%s' contains invalid value `%s'",
+      {  xprintf("%s:%d: field `%s' contains invalid value `%s'\n",
             dsa->fname, dsa->seqn, fld, str);
          return 1;
       }
       return 0;
 }
 
-/*----------------------------------------------------------------------
--- parse_fmt - parse Fortran format specification.
---
--- This routine parses the Fortran format specification represented as
--- character string which fmt points to and stores format elements into
--- appropriate static locations. Should note that not all valid Fortran
--- format specifications may be recognised. If the format specification
--- was recognised, the routine returns zero, otherwise non-zero. */
+/***********************************************************************
+*  parse_fmt - parse Fortran format specification
+*
+*  This routine parses the Fortran format specification represented as
+*  character string which fmt points to and stores format elements into
+*  appropriate static locations. Should note that not all valid Fortran
+*  format specifications may be recognised. If the format specification
+*  was recognised, the routine returns zero, otherwise non-zero. */
 
 static int parse_fmt(struct dsa *dsa, char *fmt)
 {     int k, s, val;
       char str[80+1];
       /* first character should be left parenthesis */
       if (fmt[0] != '(')
-fail: {  print("hbm_read_mat: format `%s' not recognised", fmt);
+fail: {  xprintf("hbm_read_mat: format `%s' not recognised\n", fmt);
          return 1;
       }
       k = 1;
@@ -217,14 +223,14 @@ iter:    dsa->fmt_k = val;
       return 0;
 }
 
-/*----------------------------------------------------------------------
--- read_int_array - read array of integer type.
---
--- This routine reads an integer array from the input text file, where
--- name is array name, fmt is Fortran format specification that controls
--- reading, n is number of array elements, val is array of integer type.
--- If the array was read successful, the routine returns zero, otherwise
--- non-zero. */
+/***********************************************************************
+*  read_int_array - read array of integer type
+*
+*  This routine reads an integer array from the input text file, where
+*  name is array name, fmt is Fortran format specification that controls
+*  reading, n is number of array elements, val is array of integer type.
+*  If the array was read successful, the routine returns zero, otherwise
+*  non-zero. */
 
 static int read_int_array(struct dsa *dsa, char *name, char *fmt,
       int n, int val[])
@@ -233,7 +239,8 @@ static int read_int_array(struct dsa *dsa, char *name, char *fmt,
       if (parse_fmt(dsa, fmt)) return 1;
       if (!(dsa->fmt_f == 'I' && dsa->fmt_w <= 80 &&
             dsa->fmt_k * dsa->fmt_w <= 80))
-      {  print("%s:%d: can't read array `%s' - invalid format `%s'",
+      {  xprintf(
+            "%s:%d: can't read array `%s' - invalid format `%s'\n",
             dsa->fname, dsa->seqn, name, fmt);
          return 1;
       }
@@ -246,7 +253,8 @@ static int read_int_array(struct dsa *dsa, char *name, char *fmt,
          str[dsa->fmt_w] = '\0';
          strspx(str);
          if (str2int(str, &val[k]))
-         {  print("%s:%d: can't read array `%s' - invalid value `%s'",
+         {  xprintf(
+               "%s:%d: can't read array `%s' - invalid value `%s'\n",
                dsa->fname, dsa->seqn, name, str);
             return 1;
          }
@@ -254,14 +262,14 @@ static int read_int_array(struct dsa *dsa, char *name, char *fmt,
       return 0;
 }
 
-/*----------------------------------------------------------------------
--- read_real_array - read array of real type.
---
--- This routine reads a real array from the input text file, where name
--- is array name, fmt is Fortran format specification that controls
--- reading, n is number of array elements, val is array of real type.
--- If the array was read successful, the routine returns zero, otherwise
--- non-zero. */
+/***********************************************************************
+*  read_real_array - read array of real type
+*
+*  This routine reads a real array from the input text file, where name
+*  is array name, fmt is Fortran format specification that controls
+*  reading, n is number of array elements, val is array of real type.
+*  If the array was read successful, the routine returns zero, otherwise
+*  non-zero. */
 
 static int read_real_array(struct dsa *dsa, char *name, char *fmt,
       int n, double val[])
@@ -270,7 +278,8 @@ static int read_real_array(struct dsa *dsa, char *name, char *fmt,
       if (parse_fmt(dsa, fmt)) return 1;
       if (!(dsa->fmt_f != 'I' && dsa->fmt_w <= 80 &&
             dsa->fmt_k * dsa->fmt_w <= 80))
-      {  print("%s:%d: can't read array `%s' - invalid format `%s'",
+      {  xprintf(
+            "%s:%d: can't read array `%s' - invalid format `%s'\n",
             dsa->fname, dsa->seqn, name, fmt);
          return 1;
       }
@@ -283,8 +292,8 @@ static int read_real_array(struct dsa *dsa, char *name, char *fmt,
          str[dsa->fmt_w] = '\0';
          strspx(str);
          if (strchr(str, '.') == NULL && strcmp(str, "0"))
-         {  print("%s(%d): can't read array `%s' - value `%s' has no de"
-               "cimal point", dsa->fname, dsa->seqn, name, str);
+         {  xprintf("%s(%d): can't read array `%s' - value `%s' has no "
+               "decimal point\n", dsa->fname, dsa->seqn, name, str);
             return 1;
          }
          /* sometimes lower case letters appear */
@@ -303,7 +312,8 @@ static int read_real_array(struct dsa *dsa, char *name, char *fmt,
             *ptr = 'E';
          }
          if (str2num(str, &val[k]))
-         {  print("%s:%d: can't read array `%s' - invalid value `%s'",
+         {  xprintf(
+               "%s:%d: can't read array `%s' - invalid value `%s'\n",
                dsa->fname, dsa->seqn, name, str);
             return 1;
          }
@@ -311,15 +321,16 @@ static int read_real_array(struct dsa *dsa, char *name, char *fmt,
       return 0;
 }
 
-HBM *hbm_read_mat(char *fname)
+HBM *hbm_read_mat(const char *fname)
 {     struct dsa _dsa, *dsa = &_dsa;
       HBM *hbm = NULL;
       dsa->fname = fname;
-      print("hbm_read_mat: reading matrix from `%s'...", dsa->fname);
-      dsa->fp = xfopen(dsa->fname, "r");
+      xprintf("hbm_read_mat: reading matrix from `%s'...\n",
+         dsa->fname);
+      dsa->fp = fopen(dsa->fname, "r");
       if (dsa->fp == NULL)
-      {  print("hbm_read_mat: unable to open `%s' - %s", dsa->fname,
-            strerror(errno));
+      {  xprintf("hbm_read_mat: unable to open `%s' - %s\n",
+            dsa->fname, strerror(errno));
          goto fail;
       }
       dsa->seqn = 0;
@@ -329,10 +340,10 @@ HBM *hbm_read_mat(char *fname)
       if (read_card(dsa)) goto fail;
       memcpy(hbm->title, dsa->card, 72), hbm->title[72] = '\0';
       strtrim(hbm->title);
-      print("%s", hbm->title);
+      xprintf("%s\n", hbm->title);
       memcpy(hbm->key, dsa->card+72, 8), hbm->key[8] = '\0';
       strspx(hbm->key);
-      print("key = %s", hbm->key);
+      xprintf("key = %s\n", hbm->key);
       /* read the second heading card */
       if (read_card(dsa)) goto fail;
       if (scan_int(dsa, "totcrd",  0, 14, &hbm->totcrd)) goto fail;
@@ -340,25 +351,25 @@ HBM *hbm_read_mat(char *fname)
       if (scan_int(dsa, "indcrd", 28, 14, &hbm->indcrd)) goto fail;
       if (scan_int(dsa, "valcrd", 42, 14, &hbm->valcrd)) goto fail;
       if (scan_int(dsa, "rhscrd", 56, 14, &hbm->rhscrd)) goto fail;
-      print("totcrd = %d; ptrcrd = %d; indcrd = %d; valcrd = %d; rhscrd"
-         " = %d", hbm->totcrd, hbm->ptrcrd, hbm->indcrd, hbm->valcrd,
-         hbm->rhscrd);
+      xprintf("totcrd = %d; ptrcrd = %d; indcrd = %d; valcrd = %d; rhsc"
+         "rd = %d\n", hbm->totcrd, hbm->ptrcrd, hbm->indcrd,
+         hbm->valcrd, hbm->rhscrd);
       /* read the third heading card */
       if (read_card(dsa)) goto fail;
       memcpy(hbm->mxtype, dsa->card, 3), hbm->mxtype[3] = '\0';
       if (strchr("RCP",   hbm->mxtype[0]) == NULL ||
           strchr("SUHZR", hbm->mxtype[1]) == NULL ||
           strchr("AE",    hbm->mxtype[2]) == NULL)
-      {  print("%s:%d: matrix type `%s' not recognised", dsa->fname,
-            dsa->seqn, hbm->mxtype);
+      {  xprintf("%s:%d: matrix type `%s' not recognised\n",
+            dsa->fname, dsa->seqn, hbm->mxtype);
          goto fail;
       }
       if (scan_int(dsa, "nrow", 14, 14, &hbm->nrow)) goto fail;
       if (scan_int(dsa, "ncol", 28, 14, &hbm->ncol)) goto fail;
       if (scan_int(dsa, "nnzero", 42, 14, &hbm->nnzero)) goto fail;
       if (scan_int(dsa, "neltvl", 56, 14, &hbm->neltvl)) goto fail;
-      print("mxtype = %s; nrow = %d; ncol = %d; nnzero = %d; neltvl = %"
-         "d", hbm->mxtype, hbm->nrow, hbm->ncol, hbm->nnzero,
+      xprintf("mxtype = %s; nrow = %d; ncol = %d; nnzero = %d; neltvl ="
+         " %d\n", hbm->mxtype, hbm->nrow, hbm->ncol, hbm->nnzero,
          hbm->neltvl);
       /* read the fourth heading card */
       if (read_card(dsa)) goto fail;
@@ -370,7 +381,7 @@ HBM *hbm_read_mat(char *fname)
       strspx(hbm->valfmt);
       memcpy(hbm->rhsfmt, dsa->card+52, 20), hbm->rhsfmt[20] = '\0';
       strspx(hbm->rhsfmt);
-      print("ptrfmt = %s; indfmt = %s; valfmt = %s; rhsfmt = %s",
+      xprintf("ptrfmt = %s; indfmt = %s; valfmt = %s; rhsfmt = %s\n",
          hbm->ptrfmt, hbm->indfmt, hbm->valfmt, hbm->rhsfmt);
       /* read the fifth heading card (optional) */
       if (hbm->rhscrd <= 0)
@@ -383,8 +394,8 @@ HBM *hbm_read_mat(char *fname)
          memcpy(hbm->rhstyp, dsa->card, 3), hbm->rhstyp[3] = '\0';
          if (scan_int(dsa, "nrhs", 14, 14, &hbm->nrhs)) goto fail;
          if (scan_int(dsa, "nrhsix", 28, 14, &hbm->nrhsix)) goto fail;
-         print("rhstyp = `%s'; nrhs = %d; nrhsix = %d", hbm->rhstyp,
-            hbm->nrhs, hbm->nrhsix);
+         xprintf("rhstyp = `%s'; nrhs = %d; nrhsix = %d\n",
+            hbm->rhstyp, hbm->nrhs, hbm->nrhsix);
       }
       /* read matrix structure */
       hbm->colptr = xcalloc(1+hbm->ncol+1, sizeof(int));
@@ -438,7 +449,7 @@ HBM *hbm_read_mat(char *fname)
             hbm->rhsval)) goto fail;
       }
       else
-      {  print("%s:%d: right-hand side type `%c' not recognised",
+      {  xprintf("%s:%d: right-hand side type `%c' not recognised\n",
             dsa->fname, dsa->seqn, hbm->rhstyp[0]);
          goto fail;
       }
@@ -457,8 +468,8 @@ HBM *hbm_read_mat(char *fname)
             hbm->xexact)) goto fail;
       }
 done: /* reading has been completed */
-      print("hbm_read_mat: %d cards were read", dsa->seqn);
-      xfclose(dsa->fp);
+      xprintf("hbm_read_mat: %d cards were read\n", dsa->seqn);
+      fclose(dsa->fp);
       return hbm;
 fail: /* something wrong in Danish kingdom */
       if (hbm != NULL)
@@ -472,22 +483,24 @@ fail: /* something wrong in Danish kingdom */
          if (hbm->xexact != NULL) xfree(hbm->xexact);
          xfree(hbm);
       }
-      if (dsa->fp != NULL) xfclose(dsa->fp);
+      if (dsa->fp != NULL) fclose(dsa->fp);
       return NULL;
 }
 
-/*----------------------------------------------------------------------
--- hbm_free_mat - free sparse matrix in Harwell-Boeing format.
---
--- *Synopsis*
---
--- #include "glphbm.h"
--- void hbm_free_mat(HBM *hbm);
---
--- *Description*
---
--- The hbm_free_mat routine frees all the memory allocated to the data
--- structure containing a sparse matrix in the Harwell-Boeing format. */
+/***********************************************************************
+*  NAME
+*
+*  hbm_free_mat - free sparse matrix in Harwell-Boeing format
+*
+*  SYNOPSIS
+*
+*  #include "glphbm.h"
+*  void hbm_free_mat(HBM *hbm);
+*
+*  DESCRIPTION
+*
+*  The hbm_free_mat routine frees all the memory allocated to the data
+*  structure containing a sparse matrix in the Harwell-Boeing format. */
 
 void hbm_free_mat(HBM *hbm)
 {     if (hbm->colptr != NULL) xfree(hbm->colptr);
